/*************************************************************************
 *
 *    (c) Olimex Ltd. 2010
 *
 *    File name   : UartComm.c
 *    Description : UART Communication using polling
 *
 *    History :
 *    1. Date        : 9 June 2010
 *       Author      : Aleksandar Mitev
 *       Description : Create
 *
 *  Both buffers are organized as follows:
 *	TAIL -----> HEAD
 *
 *	When writing to buffer HEAD is incremented, when reading TAIL is incremented!
 *
 **************************************************************************/
#include "includes.h"

/* Private define ------------------------------------------------------------*/
#define DESIRED_BAUDRATE    	(9600)      //The desired BaudRate
#define UART_MODULE_USED		UART1

#define RX_BUFFER_SIZE			128
#define TX_BUFFER_SIZE			128

/* Private typedef -----------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
static char rxBuffer[RX_BUFFER_SIZE];
static int rxBufHead, rxBufTail;
static BOOL rxBufEmptyFlag;

static char txBuffer[TX_BUFFER_SIZE];
static int txBufHead, txBufTail;
static BOOL txBufEmptyFlag;

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/*************************************************************************
 * Function Name: UartCommInit(..)
 * Parameters: none
 *
 * Return: none
 *
 * Description: initializes UART module and data structures
 *
 *************************************************************************/
void UartCommInit(void)
{
    UARTConfigure(UART_MODULE_USED, UART_ENABLE_PINS_TX_RX_ONLY);
    UARTSetFifoMode(UART_MODULE_USED, UART_INTERRUPT_ON_TX_NOT_FULL | UART_INTERRUPT_ON_RX_NOT_EMPTY);
    UARTSetLineControl(UART_MODULE_USED, UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1);
    UARTSetDataRate(UART_MODULE_USED, GetPeripheralClock(), DESIRED_BAUDRATE);
    UARTEnable(UART_MODULE_USED, UART_ENABLE_FLAGS(UART_PERIPHERAL | UART_RX | UART_TX));
    
    rxBufHead = rxBufTail = 0;
    rxBufEmptyFlag = TRUE;
    
    txBufHead = txBufTail = 0;
    txBufEmptyFlag = TRUE;
}

/*************************************************************************
 * Function Name: UartCommDoWork(..)
 * Parameters: none
 *
 * Return: TRUE if some work done, FALSE if called without doing smth useful
 *
 * Description: call this function periodically to send and receive data
 *
 *************************************************************************/
BOOL UartCommDoWork(void)
{
	BOOL doneWork = FALSE;
	
	// if there are any pending characters send them
	if(!txBufEmptyFlag) {
		if( UARTTransmitterIsReady(UART_MODULE_USED) ) {
			// send character here
			UARTSendDataByte(UART_MODULE_USED, txBuffer[txBufTail]);
			
			//	while(!UARTTransmissionHasCompleted(UART2))
			//	;
			
			// update pointers
			if(++txBufTail >= TX_BUFFER_SIZE)
				txBufTail = 0;
					
			if(txBufTail == txBufHead)
				txBufEmptyFlag = TRUE;
		}		
		
		doneWork = TRUE;
	}
	
	// if there is any received character receive it
	if( (rxBufHead == rxBufTail) && !rxBufEmptyFlag) {
		// nothing to do if buffer is full
	}	
	else {
		// receive here
		if( UARTReceivedDataIsAvailable(UART_MODULE_USED) ) {
		
		rxBuffer[rxBufHead] = UARTGetDataByte(UART_MODULE_USED);

		// update pointers
		rxBufEmptyFlag = FALSE;
		if(++rxBufHead >= RX_BUFFER_SIZE)
			rxBufHead = 0;
		}
		
		doneWork = TRUE;
	}
	
	return doneWork;
}

/*************************************************************************
 * Function Name: UartCommWrite(..)
 * Parameters:	pData - pointer to the buffer containing data to be send
 *				len - len in bytes of the buffer to write
 *
 * Return: The actual number of characters written to the output buffer,
 *			0 - means tx buffer is full
 *
 * Description: write data to tx buffer
 *
 *************************************************************************/
int UartCommWrite(char *pData, int len)
{	
	int count = 0;
	
	while(len--) {
		if( (txBufHead == txBufTail) && !txBufEmptyFlag) {
			break;
		}	
		else {
			txBufEmptyFlag = FALSE;
			txBuffer[txBufHead++] = *pData++;
			if(txBufHead >= TX_BUFFER_SIZE)
				txBufHead = 0;
				
			count++;
		}
	}
	
	return count;
}

/*************************************************************************
 * Function Name: UartCommWriteExact(..)
 * Parameters:	pData - pointer to the buffer containing data to be send
 *				len - len in bytes of the buffer to write
 *
 * Return:	 0 - means all needed data written sucessfuly
 *			-1 - not enough space
 *			-2 - or some error occured
 *
 * Description: write data to tx buffer, if not enough space is available, return error
 *
 *************************************************************************/
int UartCommWriteExact(char *pData, int len)
{	
	int freeSpace, count;
	
	if(txBufEmptyFlag)
		freeSpace = TX_BUFFER_SIZE;
	else {
		if(txBufHead > txBufTail)
			freeSpace = TX_BUFFER_SIZE - (txBufHead - txBufTail);
		else
			freeSpace = txBufTail - txBufHead;
	}
	
	if(len > freeSpace)
		return -2;
	
	while(len--) {
		if( (txBufHead == txBufTail) && !txBufEmptyFlag) {
			break;
		}	
		else {
			txBufEmptyFlag = FALSE;
			txBuffer[txBufHead++] = *pData++;
			if(txBufHead >= TX_BUFFER_SIZE)
				txBufHead = 0;
		}
	}
	
	return (len == -1) ? 0 : -1;
}

/*************************************************************************
 * Function Name: UartCommRead(..)
 * Parameters:	pData - pointer to the buffer that will contain the outputted data
 *				len - number of bytes that are needed
 *
 * Return: The actual number of characters read
 *			0 - means rx buffer is empty
 *
 * Description: read data from rx buffer
 *
 *************************************************************************/
int UartCommRead(char *pData, int len)
{
	int count = 0;
	
	while(len--) {
		if(rxBufEmptyFlag) {
			break;
		}	
		else {
			*(pData + count++) = rxBuffer[rxBufTail++];
			if(rxBufTail >= RX_BUFFER_SIZE)
				rxBufTail = 0;
				
			if(rxBufHead == rxBufTail)
				rxBufEmptyFlag = TRUE;
		}
	}
	
	return count;
}

/*************************************************************************
 * Function Name: UartCommReadExact(..)
 * Parameters:	pData - pointer to the buffer that will contain the outputted data
 *				len - number of bytes that are needed
 *
 * Return:	 0 - means all needed data read sucessfuly
 *			-1 - not enough space
 *			-2 - or some error occured
 *
 * Description: read data from rx buffer, if not enough data in rxBuffer, return with error code!
 *
 *************************************************************************/
int UartCommReadExact(char *pData, int len)
{
	int freeSpace, count;
	
	if(rxBufEmptyFlag)
		freeSpace = RX_BUFFER_SIZE;
	else {
		if(rxBufHead > rxBufTail)
			freeSpace = RX_BUFFER_SIZE - (rxBufHead - rxBufTail);
		else
			freeSpace = rxBufTail - rxBufHead;
	}
	
	if(len > freeSpace)
		return -2;

	while(len--) {
		if(rxBufEmptyFlag) {
			break;
		}	
		else {
			*pData++ = rxBuffer[rxBufTail++];
			if(rxBufTail >= RX_BUFFER_SIZE)
				rxBufTail = 0;
				
			if(rxBufHead == rxBufTail)
				rxBufEmptyFlag = TRUE;
		}
	}
	
	return (len == -1) ? 0 : -1;
}


